@@ -1,4 +1,3 @@
-
 /** -*- Mode: C++; tab-width: 4 -*-
  * vim: sw=4 ts=4:
  *
@@ -26,13 +25,16 @@
  *
  **/
 
+#include "config.h"
+
 #include <getopt.h>
-#include <config.h>
-#include <gtk/gtk.h>
-#include <gksuui.h>
-#include <gksu.h>
 #include <sys/stat.h>
 
+#include <gtk/gtk.h>
+#include <libgksuui1.0/gksuui-convenience.h>
+#include <libgksuui1.0/gksuui.h>
+#include <libgksu1.2/gksu.h>
+
 #include "gdeb.h"
 
 #include <apt-pkg/debfile.h>
@@ -110,20 +112,20 @@ DebPackage::DebPackage (const string& name) : filename (name), fd (0), df (0) {
 	fd = new FileFd (name, FileFd::ReadOnly);
 	if (_error->PendingError()) {
 		_error->DumpErrors();
-		_error->Error (_("Failure opening package `%s' for read."), name.c_str());
+		_error->Error (_("Failure opening package `%s' for reading."), name.c_str());
 		throw gdebError();
 	}
 
 	df = new debDebFile (*fd);
 	if (!ctrl.Read (*df)) {
 		_error->DumpErrors();
-		_error->Error (_("Failure reading the package's contents."));
+		_error->Error (_("Failure reading the package `%s''s contents."), name.c_str());
 		throw gdebError();
 	}
 
 	if (!ctrl.Control) {
 		_error->DumpErrors();
-		_error->Error (_("Could not extract the package's control record."));
+		_error->Error (_("Could not extract the control record from package `%s'."), name.c_str());
 		throw gdebError();
 	}
 }
@@ -225,10 +227,17 @@ DebPackage::Maintainer (void) {
 string
 DebPackage::Filelist (void) {
 	if (df) {
+		const gchar* decprg = "gzip";
 		const ARArchive::Member* data = df->GotoMember ("data.tar.gz");
-		if (!data) return "";
+		if (!data) {
+			data = df->GotoMember ("data.tar.bz2");
+			decprg = "bzip2";
+		}
+		if (!data) {
+			return "";
+		}
 
-		ExtractTar tpart (df->GetFile(), data->Size);
+		ExtractTar tpart (df->GetFile(), data->Size, decprg);
 
 		ListStream list;
 		if (!tpart.Go (list)) return "";
@@ -261,41 +270,24 @@ gdeb_response_cb (GtkWidget* wdg, gint responseid, gpointer data) {
 	if (responseid == GTK_RESPONSE_CLOSE) {
 		gtk_main_quit();
 	} else if (responseid == GDEB_RESPONSE_INSTALL) {
-	  GtkWidget *dialog;
-	  GksuContext *context;
-	  int retvalue = 0;
-	  gchar *password = NULL;
-	  GError *error = NULL;
+		gchar* cmd = g_strdup_printf (_("x-terminal-emulator -e /bin/sh -c " \
+		      "\\\"/usr/bin/dpkg -i %s && echo Installation done || echo Installation failed;" \
+		      "echo Press enter to continue; read DISCARDED\\\""),
+		      ((DebPackage*) data)->Filename().c_str());
+		GksuContext* ctx = gksu_context_new();
+		gksu_context_set_command (ctx, cmd);
+		g_free (cmd);
 
-	  gchar* msg = g_strdup_printf (_("Gdeb needs superuser priviledges to install %s."),
-									((DebPackage*) data)->Name().c_str());
+		gchar* msg = g_strdup_printf (_("Gdeb needs superuser priviledges to install %s."),
+		      ((DebPackage*) data)->Name().c_str());
+		gchar* pwd = gksu_ask_password (_("Enter root password"), msg, FALSE);
+		gksu_context_set_password (ctx, pwd);
+		memset (pwd, 0, strlen (pwd));
+		g_free (msg);
+		g_free (pwd);
 
-	  dialog = gksuui_dialog_new ();
-	  context = gksu_context_new ();
-	  
-	  gksuui_dialog_set_message (GKSUUI_DIALOG(dialog), msg);
-	  g_free (msg);
-
-	  gtk_window_set_title (GTK_WINDOW(dialog), _("Enter root password"));
-	  retvalue = gtk_dialog_run (GTK_DIALOG(dialog));
-	  gtk_widget_hide (dialog);
-  
-	  msg = g_strdup_printf (_("x-terminal-emulator -e /bin/sh -c " \
-							   "\\\"/usr/bin/dpkg -i %s && echo Installation done || echo Installation failed;" \
-							   "echo Press enter to continue; read DISCARDED\\\""),
-							 ((DebPackage*) data)->Filename().c_str());
-	  if (retvalue != GTK_RESPONSE_OK)
-		return;
-
-      password = gksuui_dialog_get_password (GKSUUI_DIALOG(dialog));
-	  gksu_context_set_command (context, msg);
-	  gksu_context_set_password (context, password);
-	  gksu_context_run (context, &error);
-	  if(error) {
-		printf("error:\n command: %s\n user: %s\n",
-				 gksu_context_get_command (context),
-				 gksu_context_get_user (context));
-	  }
+		gksu_context_run (ctx, NULL);
+		//gksu_context_free (ctx);
 	}
 }
 
@@ -320,8 +312,8 @@ gdeb_usage (GtkMessageType msgtype = GTK_MESSAGE_INFO) {
 	GtkWidget* w = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0,
 	      msgtype, GTK_BUTTONS_CLOSE,
 	      _("Usage:\n"
-			"  gdeb -I | --info <deb>\tShow information about package\n"
-	        "  gdeb -h | --help\tDisplay this help and exit"));
+			"  " GDEB_NAME " -I | --info <deb>\tShow information about package\n"
+	        "  " GDEB_NAME " -h | --help\tDisplay this help and exit"));
 	gtk_dialog_set_has_separator (GTK_DIALOG (w), FALSE);
 	gtk_dialog_run (GTK_DIALOG (w));
 
@@ -349,9 +341,9 @@ int main ( int argc, char ** argv )
 			gdeb_usage();
 			exit (EXIT_SUCCESS);
 		} else if (optc == '?') {
-			gdeb_printerr (_("gdeb: unknown option -%c"), optopt);
+			gdeb_printerr (_(GDEB_NAME ": unknown option -%c"), optopt);
 		} else if (optc == ':') {
-			gdeb_printerr (_("gdeb: missing option argument"));
+			gdeb_printerr (_(GDEB_NAME ": missing option argument"));
 		} else if (optc != 'I') {
 			gdeb_printerr (_("The option parser is acting up!?"));
 		}
